@@ -17,9 +17,55 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "gstffmpegenc.h"
-
 #include <string.h>
+#include <assert.h>
+#include "config.h"
+#ifdef HAVE_FFMPEG_UNINSTALLED
+#include <avcodec.h>
+#else
+#include <ffmpeg/avcodec.h>
+#endif
+
+#include <gst/gst.h>
+
+typedef struct _GstFFMpegEnc GstFFMpegEnc;
+
+struct _GstFFMpegEnc {
+  GstElement element;
+
+  /* We need to keep track of our pads, so we do so here. */
+  GstPad *srcpad;
+  GstPad *sinkpad;
+
+  gboolean need_resample;
+  gint in_width, in_height;
+  gint out_width, out_height;
+
+  guchar *buffer;
+  gint buffer_pos;
+
+  AVCodecContext *context;
+  ImgReSampleContext *resample;
+};
+
+typedef struct _GstFFMpegEncClass GstFFMpegEncClass;
+
+struct _GstFFMpegEncClass {
+  GstElementClass parent_class;
+
+  AVCodec *in_plugin;
+};
+
+#define GST_TYPE_FFMPEGENC \
+  (gst_ffmpegenc_get_type())
+#define GST_FFMPEGENC(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_FFMPEGENC,GstFFMpegEnc))
+#define GST_FFMPEGENC_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_FFMPEGENC,GstFFMpegEncClass))
+#define GST_IS_FFMPEGENC(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_FFMPEGENC))
+#define GST_IS_FFMPEGENC_CLASS(obj) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_FFMPEGENC))
 
 #define VIDEO_BUFFER_SIZE (1024*1024)
 
@@ -168,7 +214,7 @@ gst_ffmpegenc_class_init (GstFFMpegEncClass *klass)
                       0, G_MAXINT, 128000, G_PARAM_READWRITE)); 
     g_object_class_install_property(G_OBJECT_CLASS (klass), ARG_SAMPLE_RATE,
       g_param_spec_int ("sample_rate","rate","rate",
-                      0, G_MAXINT, -1, G_PARAM_READWRITE)); 
+                      -1, G_MAXINT, -1, G_PARAM_READWRITE)); 
     g_object_class_install_property(G_OBJECT_CLASS (klass), ARG_HQ,
       g_param_spec_boolean ("hq","hq","hq",
                       FALSE, G_PARAM_READWRITE)); 
@@ -178,14 +224,14 @@ gst_ffmpegenc_class_init (GstFFMpegEncClass *klass)
   gobject_class->get_property = gst_ffmpegenc_get_property;
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_ffmpegenc_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstFFMpegEnc *ffmpegenc = (GstFFMpegEnc *) gst_pad_get_parent (pad);
   GstFFMpegEncClass *oclass = (GstFFMpegEncClass*)(G_OBJECT_GET_CLASS(ffmpegenc));
 
   if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 
   if (strstr (gst_caps_get_mime (caps), "audio/raw")) {
     gst_caps_get_int (caps, "rate", &ffmpegenc->context->sample_rate);
@@ -218,12 +264,12 @@ gst_ffmpegenc_sinkconnect (GstPad *pad, GstCaps *caps)
   }
   else {
     g_warning ("ffmpegenc: invalid caps %s\n", gst_caps_get_mime (caps));
-    return GST_PAD_CONNECT_REFUSED;
+    return GST_PAD_LINK_REFUSED;
   }
 
   if (avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0) {
     g_warning ("ffmpegenc: could not open codec\n");
-    return GST_PAD_CONNECT_REFUSED;
+    return GST_PAD_LINK_REFUSED;
   }
 
   if (oclass->in_plugin->type == CODEC_TYPE_AUDIO) {
@@ -231,7 +277,7 @@ gst_ffmpegenc_sinkconnect (GstPad *pad, GstCaps *caps)
 		  ffmpegenc->context->channels);
     ffmpegenc->buffer_pos = 0;
   }
-  return GST_PAD_CONNECT_OK;
+  return GST_PAD_LINK_OK;
 }
 
 static void
@@ -239,7 +285,7 @@ gst_ffmpegenc_init(GstFFMpegEnc *ffmpegenc)
 {
   GstFFMpegEncClass *oclass = (GstFFMpegEncClass*)(G_OBJECT_GET_CLASS (ffmpegenc));
 
-  ffmpegenc->context = g_malloc0 (sizeof (AVCodecContext));
+  ffmpegenc->context = avcodec_alloc_context();
 
   if (oclass->in_plugin->type == CODEC_TYPE_VIDEO) {
     ffmpegenc->sinkpad = gst_pad_new_from_template (
@@ -263,7 +309,7 @@ gst_ffmpegenc_init(GstFFMpegEnc *ffmpegenc)
     ffmpegenc->context->sample_rate = -1;
   }
 
-  gst_pad_set_connect_function (ffmpegenc->sinkpad, gst_ffmpegenc_sinkconnect);
+  gst_pad_set_link_function (ffmpegenc->sinkpad, gst_ffmpegenc_sinkconnect);
   gst_element_add_pad (GST_ELEMENT (ffmpegenc), ffmpegenc->sinkpad);
 
   ffmpegenc->srcpad = gst_pad_new_from_template (
@@ -337,7 +383,7 @@ gst_ffmpegenc_chain_video (GstPad *pad, GstBuffer *inbuf)
   GstFFMpegEnc *ffmpegenc = (GstFFMpegEnc *)(gst_pad_get_parent (pad));
   gpointer data;
   gint size, frame_size;
-  AVPicture picture, rpicture, *toencode;
+  AVFrame picture, rpicture, *toencode;
   gboolean free_data = FALSE, free_res = FALSE;
 
   data = GST_BUFFER_DATA (inbuf);
@@ -365,7 +411,7 @@ gst_ffmpegenc_chain_video (GstPad *pad, GstBuffer *inbuf)
   }
   */
 
-  avpicture_fill (&picture, data, PIX_FMT_YUV420P, ffmpegenc->in_width, ffmpegenc->in_height);
+  avpicture_fill ((AVPicture*)(void *)&picture, data, PIX_FMT_YUV420P, ffmpegenc->in_width, ffmpegenc->in_height);
   toencode = &picture;
 
   if (ffmpegenc->need_resample) {
@@ -373,12 +419,13 @@ gst_ffmpegenc_chain_video (GstPad *pad, GstBuffer *inbuf)
     guint8 *rdata;
 
     rdata = g_malloc ((rframe_size * 3)/2);
-    avpicture_fill (&rpicture, rdata, PIX_FMT_YUV420P, ffmpegenc->context->width, ffmpegenc->context->height);
+    avpicture_fill ((AVPicture*)(void *)&rpicture, rdata, PIX_FMT_YUV420P, ffmpegenc->context->width, ffmpegenc->context->height);
 
     free_res = TRUE;
     toencode = &rpicture;
     
-    img_resample (ffmpegenc->resample, &rpicture, &picture);
+    img_resample (ffmpegenc->resample, (AVPicture*)(void *)&rpicture,
+		    (AVPicture*)(void *)&picture);
   }
 
   outbuf = gst_buffer_new ();
@@ -515,7 +562,7 @@ gst_ffmpegenc_register (GstPlugin *plugin)
       goto next;
     }
     /* construct the type */
-    type_name = g_strdup_printf("ffmpeg%s_%s", codec_type, in_plugin->name);
+    type_name = g_strdup_printf("ff%s_%s", codec_type, in_plugin->name);
 
     /* if it's already registered, drop it */
     if (g_type_from_name(type_name)) {
@@ -523,8 +570,9 @@ gst_ffmpegenc_register (GstPlugin *plugin)
       goto next;
     }
 
-    /* create the gtk type now */
+    /* create the glib type now */
     type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
+    g_return_val_if_fail(type != 0, FALSE);
 
     /* construct the element details struct */
     details = g_new0 (GstElementDetails,1);
