@@ -74,6 +74,8 @@ static void		gst_xvideosink_set_property	(GObject *object, guint prop_id,
 static void		gst_xvideosink_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
 
+static void		gst_xvideosink_reset 		(GstXVideoSink *xvideosink);
+
 /* these functions require the lock xvideosink->lock to be held */
 static gboolean		gst_xvideosink_xwindow_new	(GstXVideoSink *xvideosink);
 static void		gst_xvideosink_xwindow_free	(GstXVideoSink *xvideosink);
@@ -136,7 +138,7 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
                       G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAMES_DISPLAYED,
     g_param_spec_int ("frames_displayed", "Frames Displayed", "The number of frames displayed so far",
-                      G_MININT,G_MAXINT, 0, G_PARAM_READWRITE));
+                      0,G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAME_TIME,
     g_param_spec_int ("frame_time", "Frame time", "The interval between frames",
                       G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
@@ -449,16 +451,12 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
   gst_pad_set_getcaps_function (xvideosink->sinkpad, gst_xvideosink_getcaps);
   gst_pad_set_bufferpool_function (xvideosink->sinkpad, gst_xvideosink_get_bufferpool);
 
+  /* one-time initialization */
   xvideosink->window = NULL;
   xvideosink->toplevel = TRUE;
-  xvideosink->width = 100;
-  xvideosink->height = 100;
   xvideosink->format = 0;
   xvideosink->formats = NULL;
-  xvideosink->lock = g_mutex_new();
-  xvideosink->disable_xv = FALSE;
-  xvideosink->auto_size = TRUE;
-  xvideosink->correction = 0;
+  xvideosink->lock = g_mutex_new ();
 
   xvideosink->image_pool = NULL;
   xvideosink->pool_lock = g_mutex_new ();
@@ -467,10 +465,25 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
   xvideosink->need_new_window = FALSE;
 
   xvideosink->image = NULL;
+
+  xvideosink->width = 100;
+  xvideosink->height = 100;
+  xvideosink->disable_xv = FALSE;
+  xvideosink->auto_size = TRUE;
+
+  gst_xvideosink_reset (xvideosink);
   
   GST_FLAG_SET(xvideosink, GST_ELEMENT_THREAD_SUGGESTED);
 }
 
+/* reset properties for first use or -> NULL transition */
+static void
+gst_xvideosink_reset (GstXVideoSink *xvideosink)
+{
+  xvideosink->correction = 0;
+  xvideosink->frames_displayed = 0;
+}
+
 static void
 gst_xvideosink_set_clock (GstElement *element, GstClock *clock)
 {
@@ -756,6 +769,7 @@ gst_xvideosink_change_state (GstElement *element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      gst_xvideosink_reset (xvideosink);
       break;
     case GST_STATE_READY_TO_NULL:
       gst_xvideosink_xwindow_free (xvideosink);
